var __accessCheck = (obj, member, msg) => {
  if (!member.has(obj))
    throw TypeError("Cannot " + msg);
};
var __privateGet = (obj, member, getter) => {
  __accessCheck(obj, member, "read from private field");
  return getter ? getter.call(obj) : member.get(obj);
};
var __privateAdd = (obj, member, value) => {
  if (member.has(obj))
    throw TypeError("Cannot add the same private member more than once");
  member instanceof WeakSet ? member.add(obj) : member.set(obj, value);
};
var __privateSet = (obj, member, value, setter) => {
  __accessCheck(obj, member, "write to private field");
  setter ? setter.call(obj, value) : member.set(obj, value);
  return value;
};
var __privateWrapper = (obj, member, setter, getter) => {
  return {
    set _(value) {
      __privateSet(obj, member, value, setter);
    },
    get _() {
      return __privateGet(obj, member, getter);
    }
  };
};

// src/index.ts
import { dirname, relative, resolve } from "path";

// node_modules/.pnpm/@antfu+utils@0.5.2/node_modules/@antfu/utils/dist/index.mjs
function slash(str) {
  return str.replace(/\\/g, "/");
}
var Node = class {
  value;
  next;
  constructor(value) {
    this.value = value;
  }
};
var _head, _tail, _size;
var Queue = class {
  constructor() {
    __privateAdd(this, _head, void 0);
    __privateAdd(this, _tail, void 0);
    __privateAdd(this, _size, void 0);
    this.clear();
  }
  enqueue(value) {
    const node = new Node(value);
    if (__privateGet(this, _head)) {
      __privateGet(this, _tail).next = node;
      __privateSet(this, _tail, node);
    } else {
      __privateSet(this, _head, node);
      __privateSet(this, _tail, node);
    }
    __privateWrapper(this, _size)._++;
  }
  dequeue() {
    const current = __privateGet(this, _head);
    if (!current) {
      return;
    }
    __privateSet(this, _head, __privateGet(this, _head).next);
    __privateWrapper(this, _size)._--;
    return current.value;
  }
  clear() {
    __privateSet(this, _head, void 0);
    __privateSet(this, _tail, void 0);
    __privateSet(this, _size, 0);
  }
  get size() {
    return __privateGet(this, _size);
  }
  *[Symbol.iterator]() {
    let current = __privateGet(this, _head);
    while (current) {
      yield current.value;
      current = current.next;
    }
  }
};
_head = new WeakMap();
_tail = new WeakMap();
_size = new WeakMap();
var VOID = Symbol("p-void");

// src/index.ts
import _debug from "debug";
import { emptyDir, ensureDir, existsSync, unlink } from "fs-extra";
import { parse } from "node-html-parser";

// src/utils.ts
import { createHash } from "crypto";
import axios from "axios";
import { createWriteStream } from "fs-extra";
function chunkHash(text) {
  const hex = createHash("md5").update(text).digest("hex");
  return hex.slice(8, 16);
}
function isValidHttpUrl(str) {
  let url;
  try {
    url = new URL(str);
  } catch (_) {
    return false;
  }
  return url.protocol === "http:" || url.protocol === "https:";
}
async function downloadTo(url, filepath) {
  const writer = createWriteStream(filepath);
  const response = await axios({
    url,
    method: "GET",
    responseType: "stream"
  });
  response.data.pipe(writer);
  return new Promise((resolve2, reject) => {
    writer.on("finish", resolve2);
    writer.on("error", reject);
  });
}
function getElementMeta(element, scriptMetas) {
  const tagName = element.tagName;
  const url = element.getAttribute(scriptMetas[tagName].attributeName);
  return {
    tagName: element.tagName,
    url
  };
}

// src/index.ts
var defaultScriptMetas = {
  SCRIPT: {
    attributeName: "src",
    extension: ".js"
  },
  LINK: {
    attributeName: "href",
    extension: ".css"
  }
};
var debug = _debug("vite-plugin-remote-scripts");
function VitePluginRemoteScripts(options = {}) {
  const {
    assetsDir = "node_modules/.remote-scripts",
    attributeName = "data-remote-script",
    chunkPrefix = "remote-script.",
    resolveMode = "relative",
    awaitDownload = true
  } = options;
  let dir = void 0;
  let config;
  let server;
  const tasksMap = {};
  async function transform(code, id) {
    const root = parse(code);
    const elements = root.querySelectorAll(`link[${attributeName}], script[${attributeName}]`);
    if (elements.length > 0) {
      const tasks = [];
      let hasReplaced = false;
      for (const element of elements) {
        const meta = getElementMeta(element, defaultScriptMetas);
        if (!meta.url || !isValidHttpUrl(meta.url))
          continue;
        const hash = chunkPrefix + chunkHash(meta.url) + defaultScriptMetas[meta.tagName].extension;
        let filepath;
        if (config.env.PROD) {
          filepath = slash([config.root, config.build.outDir, config.build.assetsDir, hash].join("/"));
        } else {
          filepath = slash(resolve(dir, hash));
        }
        debug("detected", meta.url, hash);
        if (!existsSync(filepath) || tasksMap[filepath]) {
          if (!tasksMap[filepath]) {
            tasksMap[filepath] = (async () => {
              try {
                debug("downloading", meta.url);
                await downloadTo(meta.url, filepath);
                debug("downloaded", meta.url);
              } catch (e) {
                if (existsSync(filepath)) {
                  await unlink(filepath);
                }
                throw e;
              } finally {
                delete tasksMap[filepath];
              }
            })();
          }
          tasks.push(tasksMap[filepath]);
          if (!config.env.PROD && !awaitDownload) {
            continue;
          }
        }
        hasReplaced = true;
        const mode = typeof resolveMode === "function" ? resolveMode(id, meta.url) : resolveMode;
        let newUrl;
        if (config.env.PROD) {
          newUrl = slash([config.base + config.build.assetsDir, hash].join("/"));
        } else {
          if (mode === "relative") {
            newUrl = slash(relative(dirname(id), `${dir}/${hash}`));
            if (newUrl[0] !== ".") {
              newUrl = `./${newUrl}`;
            }
          } else {
            let path = `${dir}/${hash}`;
            if (!path.startsWith("/")) {
              path = `/${path}`;
            }
            newUrl = `/@fs${path}`;
          }
        }
        element.setAttribute(defaultScriptMetas[element.tagName].attributeName, newUrl);
      }
      if (tasks.length) {
        if (config.env.PROD || awaitDownload) {
          await Promise.all(tasks);
        } else {
          Promise.all(tasks).then(() => {
            if (server) {
              const module = server.moduleGraph.getModuleById(id);
              if (module) {
                server.moduleGraph.invalidateModule(module);
              }
            }
          });
        }
      }
      if (!hasReplaced)
        return null;
      code = root.toString();
    }
    return code;
  }
  async function configResolved(_config) {
    config = _config;
    if (config.env.PROD) {
      dir = slash(resolve(config.root, config.build.outDir, config.build.assetsDir));
    } else {
      dir = slash(resolve(config.root, assetsDir));
    }
    if (config.server.force) {
      await emptyDir(dir);
    }
    await ensureDir(dir);
  }
  return {
    name: "vite-plugin-remote-scripts",
    enforce: "post",
    configResolved,
    configureServer(_server) {
      server = _server;
    },
    transformIndexHtml: {
      enforce: "post",
      async transform(code, ctx) {
        await configResolved(config);
        return await transform(code, ctx.filename);
      }
    }
  };
}
var src_default = VitePluginRemoteScripts;
export {
  VitePluginRemoteScripts,
  src_default as default
};
